from __future__ import absolute_import, annotations
from typing import Any, Dict, Optional, Union
from urllib.parse import urlencode

import pandas as pd
import requests

from .api_config import ApiConfig
from . import common


def get_daily_snapshot(
        output_format: Optional[str] = None,
        api_key: Optional[str] = None) -> Union[pd.DataFrame, str, Dict[str, Any]]:
    """
    Get latest point-in-time snapshot of our data-driven predictions for all issuers along
    with associated metadata.

    Args:
        output_format:                  One-of 'pandas', 'json', or 'csv'.
        api_key:                        Your Migrations.ML API key.

    Returns:
        Union[DataFrame, str, Dict[str, Any]]


    """

    api_key = common.get_api_key(api_key)
    output_format = common.get_output_format(output_format)

    query_params = {
        'api_key': api_key
    }
    url = ApiConfig.endpoint_base + '/dailySnapshot?' + urlencode(query_params)

    try:

        response = requests.get(url)

    except Exception as error:
        raise ConnectionError(
            'Failed to make request to Migrations.ML API: %s', error) from error

    if not response.ok:

        raise RuntimeError(
            'Migrations.ML API returned bad response: %s', response.reason)

    reponse_body = response.json()

    if reponse_body['status'] != 'success':

        raise RuntimeError('Migrations.ML API returned bad status')

    if output_format == 'json':

        return reponse_body

    dataframe = pd.DataFrame([
        {
            **{k: v for k, v in row.items() if k not in 'transparency_factors'},
            'transparency_equity_performance': row['transparency_factors']['equity_performance'],
            'transparency_leverage': row['transparency_factors']['leverage'],
            'transparency_macro': row['transparency_factors']['macro'],
            'transparency_market_value': row['transparency_factors']['market_value'],
            'transparency_profitability': row['transparency_factors']['profitability'],
            'transparency_solvency': row['transparency_factors']['solvency'],
        }
        for row in reponse_body['results']
    ])

    if output_format == 'csv':

        return dataframe.to_csv(index=False)

    assert output_format == 'pandas'

    return dataframe



from __future__ import absolute_import, annotations
import datetime as dt

import numpy as np

from .api_config import ApiConfig


def get_api_key(api_key: str) -> str:

    if api_key is None:

        if (api_key := ApiConfig.api_key) is None:

            raise ValueError(
                'API key must be set either gloally or passed as an argument.')

    return api_key


def get_output_format(output_format: str) -> str:

    if output_format is None:

        output_format = ApiConfig.output_format

    if output_format not in {'pandas', 'json', 'csv'}:

        raise ValueError(
            'output_format must be one-of pandas, json, or csv.')

    return output_format


def validate_identifier(identifier: str, identifier_type: str):

    if not isinstance(identifier, str) or len(identifier) == 0:
        raise ValueError('Argument identifier must be non-empty str')

    if identifier_type not in {'ticker', 'lei'}:
        raise ValueError(
            'Argument identifier_type must be one-of ticker or lei')


def parse_tenor_points(tenor_points):

    if tenor_points is None:
        return None

    if isinstance(tenor_points, np.ndarray):
        if len(tenor_points.shape) != 1:
            raise ValueError(
                'When tenor_points is expressed as numpy array, dimension must be 1')
        tenor_points = tenor_points.tolist()

    if not isinstance(tenor_points, (list, tuple)):
        tenor_points = [tenor_points, ]

    if len(tenor_points) > 20:
        raise ValueError(
            'A maximum of 20 tenor points are allowed per request')

    if not all(isinstance(point, (float, int)) for point in tenor_points):
        raise ValueError(
            'All elements of argument tenor_points must be float or int')

    if any(x < 1. or x > 40. for x in tenor_points):
        raise ValueError(
            'All tenor points must be in interval [1, 40]')

    return tenor_points


def validate_dates(start_date, end_date):

    def _validate_date(date_):

        if date_ is None:
            return date_, date_

        if isinstance(date_, (dt.date, dt.datetime)):
            if isinstance(date_, dt.datetime):
                date_ = date_.date()

            return date_.strftime('%Y-%m-%d'), date_

        if isinstance(date_, str):

            try:
                date_parsed = dt.datetime.strptime(date_, '%Y-%m-%d').date()

            except ValueError as error:
                raise ValueError('date must be format %Y-%m-%d: %s', error)

            return date_, date_parsed

    start_date, start_date_parsed = _validate_date(start_date)
    end_date, end_date_parsed = _validate_date(end_date)

    if start_date is not None and end_date is not None:

        if start_date_parsed > end_date_parsed:
            raise ValueError('start_date must be <= end_date')

    return start_date, end_date



from __future__ import absolute_import, annotations
from typing import Any, Dict, Optional, Union, List
from urllib.parse import urlencode
from pprint import pprint
import datetime as dt

import pandas as pd
import numpy as np
import requests

from .api_config import ApiConfig
from . import common


def get_yield_curve(
        identifier: str,
        identifier_type: str = 'ticker',
        tenor_points: Optional[Union[float, List[float]]] = None,
        output_format: Optional[str] = None,
        api_key: Optional[str] = None) -> Union[pd.DataFrame, str, Dict[str, Any]]:
    """
    Get latest point-in-time snapshot of our data-driven predictions for all issuers along
    with associated metadata.

    Args:
        identifier:                     Identifier of issuer.
        identifier_type:                One-of 'ticker' or 'lei'.
        tenor_points:                   Value or list of values of time-to-maturity in years.
        output_format:                  One-of 'pandas', 'json', or 'csv'.
        api_key:                        Your Migrations.ML API key.

    Returns:
        Union[DataFrame, str, Dict[str, Any]]


    """

    api_key = common.get_api_key(api_key)
    output_format = common.get_output_format(output_format)
    common.validate_identifier(identifier, identifier_type)
    tenor_points = common.parse_tenor_points(tenor_points)

    query_params = {
        'api_key': api_key,
        'identifier': identifier,
        'identifier_type': identifier_type,
    }
    if tenor_points is not None:
        # pass
        query_params['tenor_points'] = ','.join(
            str(point) for point in tenor_points)
    url = ApiConfig.endpoint_base + '/yieldCurve?' + urlencode(query_params)

    try:

        response = requests.get(url)

    except Exception as error:
        raise ConnectionError(
            'Failed to make request to Migrations.ML API: %s', error) from error

    if not response.ok:

        raise RuntimeError(
            'Migrations.ML API returned bad response: %s', response.reason)

    response_body = response.json()

    if not response_body['success']:

        raise RuntimeError('Migrations.ML API returned bad status')

    if output_format == 'json':

        return response_body

    dataframe = pd.DataFrame([
        {
            **{k: v for k, v in response_body.items() if k not in {'tenor_points', 'success', 'reason'}},
            **point
        }
        for point in response_body['tenor_points']
    ])

    if output_format == 'csv':

        return dataframe.to_csv(index=False)

    assert output_format == 'pandas'

    return dataframe


def get_yield_curve_history(
        identifier: str,
        identifier_type: str = 'ticker',
        tenor_points: Optional[Union[float, List[float]]] = None,
        start_date: Union[dt.date, str] = None,
        end_date: Union[dt.date, str] = None,
        output_format: Optional[str] = None,
        api_key: Optional[str] = None) -> Union[pd.DataFrame, str, Dict[str, Any]]:
    """
    Get latest point-in-time snapshot of our data-driven predictions for all issuers along
    with associated metadata.

    Args:
        identifier:                     Identifier of issuer.
        identifier_type:                One-of 'ticker' or 'lei'.
        tenor_points:                   Value or list of values of time-to-maturity in years.
        start_date:                     Left endpoint of date interval.
        end_date:                       Right endpoint of date interval.
        output_format:                  One-of 'pandas', 'json', or 'csv'.
        api_key:                        Your Migrations.ML API key.

    Returns:
        Union[DataFrame, str, Dict[str, Any]]


    """

    api_key = common.get_api_key(api_key)
    output_format = common.get_output_format(output_format)
    common.validate_identifier(identifier, identifier_type)
    tenor_points = common.parse_tenor_points(tenor_points)
    start_date, end_date = common.validate_dates(start_date, end_date)

    query_params = {
        'api_key': api_key,
        'identifier': identifier,
        'identifier_type': identifier_type,
    }

    if tenor_points is not None:
        query_params['tenor_points'] = ','.join(
            str(point) for point in tenor_points)

    if start_date is not None:
        query_params['start_date'] = start_date

    if end_date is not None:
        query_params['end_date'] = end_date

    url = ApiConfig.endpoint_base + \
        '/yieldCurveHistory?' + urlencode(query_params)

    try:

        response = requests.get(url)

    except Exception as error:
        raise ConnectionError(
            'Failed to make request to Migrations.ML API: %s', error) from error

    if not response.ok:

        raise RuntimeError(
            'Migrations.ML API returned bad response: %s', response.reason)

    response_body = response.json()

    if not response_body['success']:

        error_msg = 'Migrations.ML API bad status: %s' % response_body['reason']
        raise RuntimeError(error_msg)

    if output_format == 'json':

        return response_body

    meta = {
        k: v
        for k, v in response_body.items()
        if k not in {'timeseries', 'success', 'reason'}
    }

    data = [
        {
            'date': record['date'],
            **meta,
            **point
        }
        for record in response_body['timeseries']
        for point in record['tenor_points']
    ]

    dataframe = pd.DataFrame(data)

    if output_format == 'csv':

        return dataframe.to_csv(index=False)

    assert output_format == 'pandas'

    return dataframe



import setuptools
# from pathlib import Path
# from os import getenv

with open('README.md', 'r') as f:
    long_description = f.read()

setuptools.setup(
    name='migrations-ml',
    version='0.0.11',
    author='Jacques Nel',
    author_email='jacques.nel@migrations.ml',
    description='Migrations.ML advanced bond risk analytics data REST API wrapper.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://migrations.ml',
    install_requires=[
        'pandas>=1.3.2',
        'requests>=2.26.0',
        'numpy>=1.21.2'
    ],
    packages=setuptools.find_packages(),
    classifiers=[
        'Programming Language :: Python :: 3.8',
        'License :: Other/Proprietary License',
        'Development Status :: 4 - Beta',
        'Operating System :: OS Independent',
        'Intended Audience :: Financial and Insurance Industry',
        'Natural Language :: English',
        'Topic :: Office/Business :: Financial'
    ],
    python_requires='>=3.8',
    test_suite='tests'
)
